Class: SpannerLib::GoBytes
- Inherits:
-
FFI::Struct
- Object
- FFI::Struct
- SpannerLib::GoBytes
- Defined in:
- lib/spannerlib/ffi.rb
Overview
GoBytes is the Ruby representation of a Go byte slice
GoBytes is the Ruby representation of a Go byte slice