Class: SpannerLib::Message
- Inherits:
-
FFI::Struct
- Object
- FFI::Struct
- SpannerLib::Message
- Defined in:
- lib/spannerlib/ffi.rb
Overview
Message is the common return type for all native functions.
Message is the common return type for all native functions.